// naumann@stce.rwth-aachen.de

#include <iostream>
#include <fstream>
#include <cmath>
using namespace std;

double eps=1e-7;

template<typename T>
T f(T x) {
  return exp(sin(pow(x,2)));
}

template<typename T>
T df_dx(T x) {
  T delta_x=1e-7;
  return (f(x+delta_x)-f(x-delta_x))/(2*delta_x);
  // return exp(sin(pow(x,2)))*cos(pow(x,2))*2*x;
}

template<typename T>
T ddf_dxx(T x) {
  T delta_x=1e-7;
  return (df_dx(x+delta_x)-df_dx(x-delta_x))/(2*delta_x);
  // return exp(sin(pow(x,2)))*cos(pow(x,2))*2*x*cos(pow(x,2))*2*x+
  // exp(sin(pow(x,2)))*(-sin(pow(x,2)))*2*x*2*x+
  // exp(sin(pow(x,2)))*cos(pow(x,2))*2;
}

template<typename T>
T minimize(T x) {
  T x_up=x+1; 
  x=x-1; 
  ofstream o("steps");
  while (fabs(x-x_up)>eps) {
    T x_mid=(x_up+x)/2; 
    o << x_mid << " " << f(x_mid) << endl;
    T dfdx=df_dx(x);
    T dfdx_mid=df_dx(x_mid);
    if (fabs(dfdx_mid)<eps) { 
      x=x_up=x_mid;
    }
    else if ((dfdx<=0&&dfdx_mid<=0)||(dfdx>=0&&dfdx_mid>=0)) {
      x=x_mid; 
    } else {
      x_up=x_mid; 
    }
  }
  return x;
}

int main() {
  double x=0.3;
  cout << "x=" << x << endl;
  cout << "f(" << x << ")=" << f(x) << endl;
  x=minimize(x);
  cout << "x=" << x << endl;
  cout << "f(" << x << ")=" << f(x) << endl;
  cout << "dfdx(" << x << ")=" << df_dx(x) << endl;
  cout << "ddfdxx(" << x << ")=" << ddf_dxx(x) << endl;
  return 0;
}


